/**


 The Edge class represents an edge in a directed graph. It has a "from" Node and
 a "to" Node.

 @author SJSU Students

 */
public class Edge 
{

	private Node fromNode;
	private Node toNode;

	/**
	 This is the constructor for the class. The inputs are the two nodes at the
	 endpoints of the Edge.
	 @param from the node at the "from" end of the edge
	 @param to the node at the "to" end of the edge
	 */
	public Edge (Node from, Node to)
	{
		fromNode = from;
		toNode = to;
	}

	/**
	 This method returns the "from" node.
	 @return the "from" node
	 */
	public Node getFromNode()
	{
		return fromNode;
	}

	/**
	 This method returns the "to" node.
	 @return the "to" node
	 */
	public Node getToNode()
	{
		return toNode;
	}

	/**
	 This method overrides the default Object.equals() method, and returns true
	 iff the nodes on the input Edge are equals() to the nodes on this Edge.
	 @param e the Edge to be compared for equality
	 @return true iff the nodes on the input Edge are equals() to the nodes on
	 this Edge
	 */
	public boolean equals(Edge e)
	{
		return (e.fromNode.equals(fromNode)) && (e.toNode.equals(toNode));
	}

	/**
	 This method returns a new Edge with the endpoints reversed.
	 @return a new Edge with the endpoints reversed
	 */
	public Edge reverse()
	{
		return new Edge(toNode, fromNode);
	}

}
