import java.util.ListIterator;

/**

 The Node class represents a node in a directed graph. It has an identifying
 number, a list of incoming and outgoing edges, and the capacity to be marked.

 @author SJSU Students

 */
public class Node 
{

	private int nodeNumber;
	private EdgeList incomingEdges;
	private EdgeList outgoingEdges;
	private boolean marked;  //Nodes can either be marked or unmarked.

	/**
	 This is the constructor for the class. Its only input is the number by
	 which to identify the Node. Both edge lists are initially empty, and the
	 Node is initially unmarked.
	 @param number the ID number for the nodes
	 */
	public Node(int number)
	{
		nodeNumber = number;
		incomingEdges = new EdgeList();
		outgoingEdges = new EdgeList();
		marked = false;  // All nodes are initially unmarked.
	}

	/**
	 This method adds an edge to the Node's list of edges. The direction of the
	 edge is determined by looking at its endpoints.
	 @param e the edge to be added
	 */
	public void addEdge(Edge e)
	{
		EdgeList listToAddTo = pickList(e);

		if (listToAddTo == null)
		{
			System.out.println("Error in Node.addEdge(): input edge does not "
					+ "contain this node.");
			return;
		}

		if (!listToAddTo.contains(e))
			listToAddTo.add(e);
	}

	/**
	 This method removes an edge from the Node's list of edges.
	 @param e the edge to be removed
	 */
	public void removeEdge(Edge e)
	{
		EdgeList listToAddTo = pickList(e);

		if (listToAddTo == null)
		{
			System.out.println("Error in Node.removeEdge(): input edge does not"
					+ " contain this node.");
			return;
		}

		listToAddTo.remove(e);
	}

	/**
	 This method chooses the edge list (either incoming or outgoing) to which
	 the input edge belongs.
	 @param e the edge to be checked
	 @return the list to which e belongs
	 */
	private EdgeList pickList(Edge e)
	{
		if (e.getFromNode().equals(this))
			return outgoingEdges;
		else if (e.getToNode().equals(this))
			return incomingEdges;
		else
			return null;
	}

	/**
	 This method returns a ListIterator object which iterates through the list
	 of the Node's outgoing edges.
	 @return the iterator of outgoing edges
	 */
	public ListIterator outgoingEdgesIterator()
	{
		return outgoingEdges.listIterator();
	}

	/**
	 This method returns the ID number of the Node.
	 @return the ID number of the Node
	 */
	public int getNumber()
	{
		return nodeNumber;
	}

	/**
	 This method overrides the default Object.equals() method, and returns true
	 iff the input Node has the same ID number as this Node.
	 @param n the Node to be compared for equality
	 @return true iff the input node has the same ID number as this Node
	 */
	public boolean equals(Node n)
	{
		return (n.nodeNumber == nodeNumber);
	}

	/**
	 This method marks this Node.
	 */
	public void mark()
	{
		marked = true;
	}

	/**
	 This method unmarks this Node.
	 */
	public void unmark()
	{
		marked = false;
	}

	/**
	 This method returns true iff this Node is marked.
	 @return true iff this Node is marked
	 */
	public boolean isMarked()
	{
		return marked;
	}

}
